#!/usr/bin/env python2.1

##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
# 
##############################################################################


""" replay server """

__version__ = "$Id: rp_client.py,v 1.3 2002/02/25 18:45:41 andreas Exp $"

import os,sys,getopt,threading
import time,traceback,socket,random
from websiteloadtest.xmlrpclib import xmlrpclib
import ConfigParser


from websiteloadtest.common import Globals
Globals.CONFIG = ConfigParser.ConfigParser()
Globals.CONFIG.read(['rp_client.cfg',os.path.expanduser('~/.rp_client.cfg')])

from websiteloadtest.common import XmlRPCServer 
from websiteloadtest.client import LogUDPClient
from websiteloadtest.common.xmlrpcreply import XRR
from websiteloadtest.client import ClientHeartbeat,ClientReplayer

sys.setcheckinterval(500)

running = 1


def readConfig(env):

    import socket

    CLIENT_PORT = Globals.CONFIG.getint('client','port')
    SERVER_PORT = Globals.CONFIG.getint('server','port')
    SERVER_HOST = Globals.CONFIG.get('server','host')

    h = socket.gethostname()
    if h.find('.')>-1:
        CLIENT_HOST = h.split('.')[0]
    else:
        CLIENT_HOST = h

    Globals.CLIENT_HOST = env['CLIENT_HOST'] = CLIENT_HOST
    Globals.CLIENT_PORT = env['CLIENT_PORT'] = CLIENT_PORT
    Globals.SERVER_HOST = env['SERVER_HOST'] = SERVER_HOST
    Globals.SERVER_PORT = env['SERVER_PORT'] = SERVER_PORT
    Globals.CLIENT_ALIAS = Globals.CONFIG.get('client','alias')


class handlerClientServer(XmlRPCServer.xmlrpcHandler):

    def die(self):
        """ request to die """

        global running 
        Globals.LOG.log_warn("got request to die")
        running = 0        

        XRR(1,'Done')

    def heartbeat(self):
        """ heartbeat function called by rp_server """
        return XRR(1,time.time())


    def get_replayer_status(self):
        
        if Globals.REPLAYER.isSuspended():
            return 'suspended'
        else:
            return 'running'


    def start_replay(self):
        """ start relay mechanism """

        if Globals.REPLAYER.isSuspended():

            Globals.LOG.log_info("entering replay mode")

            Globals.REPLAYER.resume()
    
            return XRR(1,'Done')

        else:
            return XRR(0,'Replayer already working')



    def stop_replay(self):
        """ stop replay mechanism """
    
        if not Globals.REPLAYER.isSuspended():

            Globals.LOG.log_info("leaving replay mode")

            Globals.REPLAYER.suspend()
        
            return XRR(1,'Done')

        else:

            return XRR(0,'Replayer is already suspended')


    def uptime(self):
        import popen2

        sout,sin = popen2.popen2('uptime')
        uptime = sout.read()

        return XRR(1,uptime)


class ClientServer(XmlRPCServer.XmlRPCServer):

    def __init__(self, port):
        XmlRPCServer.XmlRPCServer.__init__(self, port, handlerClientServer)


def init():

    Globals.COOKIE = str(random.randrange(0,9e8)) + \
             str(random.randrange(0,9e8))

    # Replay Server
    Globals.rp_server = xmlrpclib.Server("http://%s:%d" % (SERVER_HOST,SERVER_PORT))
    print 'Connected to server %s:%d' % (SERVER_HOST,SERVER_PORT)

    # Our XML RPC Server
    Globals.REPLAY_XMLRPC = ClientServer(CLIENT_PORT)
    Globals.REPLAY_XMLRPC.start()

    # register ourself at the replay server
    try:
        print CLIENT_HOST,CLIENT_PORT


        Globals.rp_server.REGISTRY.register_client(CLIENT_HOST,CLIENT_PORT,Globals.COOKIE,Globals.CLIENT_ALIAS)

        res = Globals.rp_server.getConfig()
        for k,v in  res['msg'].items():
            setattr(Globals,k,v)
        
    except:
        traceback.print_exc()
        print 
        print "Error registring client"
        raise

    Globals.LOG = LogUDPClient.Logger(Globals.LOG_SERVER_HOST,Globals.LOG_SERVER_PORT)

    Globals.LOG.log_info("client registered",p=1)
    Globals.LOG.log_info('XMLRPC server running on port %d' %  CLIENT_PORT,p=1)

    # Replay thread
    Globals.REPLAYER = ClientReplayer.ClientReplayer(Globals.rp_server,name='replayer')
    Globals.REPLAYER.start()

    # Thread to check replay server
    Globals.HEARTBEAT = ClientHeartbeat.ClientHeartbeat(Globals.rp_server,Globals.COOKIE)
    Globals.HEARTBEAT.start()

def main():

    while running: time.sleep(0.1)


def shutdown():

    try:
        Globals.rp_server.REGISTRY.unregister_client(CLIENT_HOST,COOKIE)    
    except: pass

    try:
        # Hack
        Globals.LOG.log_info("client unregistered:",CLIENT_HOST,CLIENT_PORT,p=1)
    except:
        print 'client unregistered'

    
    for x in threading.enumerate():

        if x != threading.currentThread():
            print "Trying to shutdown thread",x
            x.stop()
            x.join()


def usage():

    print "%s [options] " % os.path.basename(sys.argv[0])
    print
    print "Options:"
    print "    -m|--host <hostname>    Host where the rp_server runs"
    print "    -p|--port <port>        Port number of the rp_server"
    print
    print "Version:",__version__


if __name__=='__main__':

    readConfig( globals() )

    opts,args = getopt.getopt(sys.argv[1:],'hm:p:', ['help','host=','port='])

    for k,v in opts:
        if k in ['--help','-h']:        usage(); sys.exit(1) 
        if k in ['--port','-p']:        SERVER_PORT = int(v)
        if k in ['--host','-m']:        SERVER_HOST = v

    try:
        init()
    except:
        traceback.print_exc()
        shutdown()
        os._exit(1)

    try:
        main()
    except KeyboardInterrupt:
        print "Seems, I should die :-("
    except:
        traceback.print_exc ()

    shutdown()
    os._exit(0)

