#!/usr/bin/env python2.1

##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
# 
##############################################################################


""" request object class """

__version__ = "$Id: Request.py,v 1.5 2002/02/25 18:49:07 andreas Exp $"

import sys, time
from types import DictType,ListType
import myhttplib 
from urllib import getproxies
from urlparse import urlparse


NS_headers = {
    'accept':'text/html, text/plain, image/gif,'
                            ' image/x-xbitmap, image/jpeg, image/pjpeg,'
                            ' impage/png, */*',
    'accept-encoding':'gzip',
    'accept-language':'en',
    'accept-charset':'iso-8859-1,*,utf-8',
#    'connection':'Keep-Alive',
    'user-agent':'Mozilla/4.77 [en] Zope (Windows NT 5.0; U)'
}

MSIE_headers =  {
    'accept':'text/html, text/plain,'
        ' image/gif, image/x-xbitmap, image/jpeg,'
        ' image/pjpeg, impage/png, */*',
    'accept-encoding':'gzip,deflate',
    'accept-language':'en-us',
    'accept-charset':'iso-8859-1,*,utf-8',
#    'connection':'Keep-Alive',
    'user-agent':'Mozilla/4.0  (compatible; MSIE 6.0, Windows NT 5.1)'
}

class Request:
    """ base request class """

    __attrs__ = ('timestamp','path','method','data','useragent','cond_get',
                    'throttle','headers','http_debug','outputfile')

    #
    # timestamp:  float
    #
    # path -- is either a complete URL (http://...) or path (/blabla/sux.gif).
    #         If path does not contain host information then the HOST header  
    #         must be present to determine the host
    #
    # method -- GET, POST
    #
    # data -- POST data
    #
    # useragent -- either 'MSIE' or 'NS'
    #
    # cond_get  -- 1 or 0 to emulate a conditional GET request
    #
    # throttle  -- float to emulate a slow connection. 
    #
    # headers   -- Dictionary that keeps additional headers to be send
    #
    # http_debug -- 1 or 0 to enable/disable debug
    #
    # outputfile -- filename that keeps the received data or None
    

    timestamp  = 0
    path       = '/'
    method     = 'GET'
    data       = ''
    useragent  = ''
    cond_get   = 0
    throttle   = 0
    headers    = {}
    http_debug = 0
    outputfile = None

    host       = None
    port       = None

    def __init__(self,*args,**kw):

        for k,v in kw.items():
            if not k in self.__attrs__:
                raise RuntimeError,'Unsupported keyword parameter: %s' % k
 
            setattr(self,k,v)

        # try to determine the host/port either from the URL
        # or the HOST header

        fields = urlparse(self.path)

        if fields[1]:
            # complete URL 
            if fields[1].find(":")>-1:
                self.host,self.port = fields[1].split(':')
            else:
                self.host,self.port = (fields[1], '80')

            self.path = fields[2]+'?' + fields[4]
        else:
            # try to sniff into the HOST header


            if self.headers.has_key('host'):
                v = self.headers['host'].strip()

                if v.find(":")>-1:
                    self.host,self.port = v.split(":")
                else:
                    self.host,self.port = (v,'80')

            else:
                raise RuntimeError,'host/port could not be determined'




    def setPostData(self,d):   
        """ set data for POST request """
        
        self.data = d


    def __str__(self):
        """ string representation of request """

        s = "Request(timestamp=%5.1lf,path='%s'"         % (self.timestamp,self.path)
        if self.method !="GET":  s+=",method='%s'"    % self.method
        if self.useragent:  s+=",useragent='%s'"      % self.useragent
        if self.cond_get:   s+=",cond_get=%d"         % self.cond_get
        if self.throttle:   s+=",throttle=%5.3lf"     % self.throttle
        if self.headers:    s+=",headers=%s"          % str(self.headers)
        if self.data:       s+=",data=%s"             % self.data
        if self.outputfile: s+=",outputfile='%s'"     % self.outputfile
        s+=')'

        return s

    __repr__ = __str__


    def str_short(self):
        """ return a short representation of the request """

        return '%s http://%s:%s%s' % \
            (self.method,self.host,self.port,self.path)


    def getBody(self,fullurl=0):

        body = []
        headers = {}
        a = body.append


        if self.useragent:
            if self.useragent=='MSIE':
                headers = MSIE_headers.copy()
            elif self.useragent=="NS":
                headers = MSIE_headers.copy()
            else:
                raise RuntimeError,"Unsupported useragent: %s" %\
                        self.useragent

        for k,v in self.headers.items():
            headers[ k.lower() ] = v

        for k,v in headers.items():
            a('%s: %s' % (k,v))

        if self.cond_get ==1:
            a('if-modified-since: %s' % time.asctime())        

        a('')
        a('')

        if self.method=='POST':
            try:
                a(eval(self.data))
            except:
                a('data')

        if fullurl:
            
            body.insert(0,'%s http://%s:%s/%s HTTP/1.0' % \
                (self.method,headers['host'], self.path))
        else:
            body.insert(0,'%s %s HTTP/1.0' % (self.method, self.path))

        return '\r\n'.join(body)


if __name__=='__main__':

#    r = Request(path='http://www.heise.de/sux?a=b',useragent='NS',throttle=0.0)
#    print  r.send(debug_level=0)

    r = Request(path='/sux?a=b',useragent='NS',throttle=0.0,headers={'host':'www.heise.de:80'})
    print  r.send(debug_level=0)
    print r.getBody()

