##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
# 
##############################################################################

import threading, time, Queue
import traceback,  os
from websiteloadtest.common import Globals

__version__ = '$Id: ClientReplayFeeder.py,v 1.8 2002/02/25 18:49:07 andreas Exp $'

class ClientReplayFeeder(threading.Thread,Queue.Queue):
    """ thread to put requests from a file into the
        rp_server queue
    """

    running               = 1
    suspended             = 1
    fp                    = None

    # number of clients
    numQueues             = 2           # number of clients

    # max # of request packages in queue
    queueLimit            = Globals.CONFIG.getint('feeder','maxpackages')

    # number of requests for one client
    numRequestsQueue      = Globals.CONFIG.getint('feeder','requestsperpackage')


    def __init__(self,name='clientreplayfeeder'):

        Queue.Queue.__init__(self)

        threading.Thread.__init__(self)
        self.setName(name)


    def setQueueLimit(self,queueLimit):
        self.queueLimit = queueLimit

    def setQueues(self,numQueues):
        self.numQueues = numQueues

    def setRequestsQueue(self,numRequestsQueue):
        self.numRequestsQueue = numRequestsQueue


    def run(self):

        while self.running:

            if not self.isSuspended():

                if self.qsize() < self.queueLimit and self.fp:

                    try:
                        self.refill_queue()
                    except:
                        traceback.print_exc()

                else:
                    time.sleep(0.2)

                if self.qsize()==0:
                    Globals.LOG.log_info('Suspending feeder thread (no data, queue empty)')
                    self.suspend()
            else:
                time.sleep(1)

        Globals.LOG.log_info("%s terminated" % self.getName(), p=1)    


    def refill_queue(self):

        q = {}

        for i in range(self.numQueues):     q[i] = []
            
        for i in range(self.numQueues*self.numRequestsQueue):

            if self.fp:

                line = self.fp.readline().strip()
                
                if line:
                    q[ i % self.numQueues ].append(line)


        if self.fp:
            fpos =  self.fp.tell()
            print '%5.2f %%' % (100.0 * fpos/self.flen)
        
            if fpos>=self.flen:
                self.fp = None
                Globals.LOG.log_info('End of input file reached')

                    
        for i in range(self.numQueues):  
            if len(q[i]) > 0: self.put( q[i] )




    def feedFile(self,fname):
        """ open file to used to feed the clients """

        try:
            self.fp = open(fname)
            self.flen = os.stat(fname)[6]
            return 1
        except:
            traceback.print_exc()
            self.fp = None 
            return 0


    def stop(self):            self.running = 0
    def suspend(self):         self.suspended = 1
    def resume(self):          self.suspended = 0
    def isSuspended(self):     return self.suspended


    def clear(self):
        """ clear out queue """
        Queue.Queue.__init__(self)


    def __del__(self):
        print "i am dead"
